/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WF0EL_H
#define WF0EL_H

#include "dart.h"
#include "wObserver.h"
#include "wElement.h"
#include <vector>

namespace dart
{

/**
 * @brief Scalar function to be integrated over an element
 */
class DART_API F0El : public fwk::Observer
{
public:
    F0El() : Observer() {}
    virtual ~F0El() {}

    virtual double eval(tbox::Element const &e, std::vector<double> const &phi,
                        size_t k) const = 0;
    virtual double evalGrad(tbox::Element const &e, std::vector<double> const &phi,
                            size_t k) const = 0;
};

/**
 * @brief Constant function
 */
class DART_API F0ElC : public F0El
{
    double v;

public:
    F0ElC(double _v) : F0El(), v(_v) {}
    virtual void update(double _v) override;

    virtual double eval(tbox::Element const &e, std::vector<double> const &phi,
                        size_t k) const override;
    virtual double evalGrad(tbox::Element const &e, std::vector<double> const &phi,
                            size_t k) const override;
};

/**
 * @brief Speed of sound squared
 *
 * This function computes the square of the speed of sound, and is then used
 * to compute the actual density, Mach number and pressure coefficient. This
 * function is clamped for large velocities using a Padé approximation and
 * particularized to diatomic gas. The function is normalized by its
 * freestream value.
 */
class DART_API F0ElSpeedSound : public F0El
{
protected:
    double gamma; ///< heat capacity ratio (diatomic gas only)
    double mInf;  ///< freestream Mach number
private:
    double uC;   ///< critical velocity
    double aC;   ///< critical speed of sound squared
    double beta; ///< ratio for Padé

public:
    F0ElSpeedSound(double _mInf, double _mC2 = 5);

    virtual double eval(tbox::Element const &e, std::vector<double> const &phi,
                        size_t k) const override;
    virtual double evalGrad(tbox::Element const &e, std::vector<double> const &phi,
                            size_t k) const override;
};

/**
 * @brief Isentropic density
 *
 * The density is particularized to diatomic gas. The density is normalized by
 * its freestream value.
 */
class DART_API F0ElRho : public F0El
{
    double gamma; ///< heat capacity ratio (diatomic gas only)
    double mInf;  ///< freestream Mach number

public:
    F0ElRho(double _mInf) : F0El(), gamma(1.4), mInf(_mInf) {}

    virtual double eval(tbox::Element const &e, std::vector<double> const &phi,
                        size_t k) const override;
    virtual double evalGrad(tbox::Element const &e, std::vector<double> const &phi,
                            size_t k) const override;
};

/**
 * @brief Isentropic clamped density
 *
 * The density is clamped (Padé approximation) for large values of velocity and
 * particularized to diatomic gas. The density is normalized by its
 * freestream value.
 */
class DART_API F0ElRhoClamp : public F0ElSpeedSound
{
public:
    F0ElRhoClamp(double _mInf, double _mC2 = 5) : F0ElSpeedSound(_mInf, _mC2) {}

    virtual double eval(tbox::Element const &e, std::vector<double> const &phi,
                        size_t k) const override;
    virtual double evalGrad(tbox::Element const &e, std::vector<double> const &phi,
                            size_t k) const override;
};

/**
 * @brief Linear density (constant)
 */
class DART_API F0ElRhoLin : public F0El
{
public:
    F0ElRhoLin() : F0El() {}

    virtual double eval(tbox::Element const &e, std::vector<double> const &phi,
                        size_t k) const override;
    virtual double evalGrad(tbox::Element const &e, std::vector<double> const &phi,
                            size_t k) const override;
};

/**
 * @brief Isentropic Mach number
 *
 * The Mach number is particularized to diatomic gas.
 */
class DART_API F0ElMach : public F0El
{
    double gamma; ///< heat capacity ratio (diatomic gas only)
    double mInf;  ///< freestream Mach number

public:
    F0ElMach(double _mInf) : F0El(), gamma(1.4), mInf(_mInf) {}

    virtual double eval(tbox::Element const &e, std::vector<double> const &phi,
                        size_t k) const override;
    virtual double evalGrad(tbox::Element const &e, std::vector<double> const &phi,
                            size_t k) const override;
};

/**
 * @brief Isentropic clamped Mach number
 *
 * The Mach number is clamped (Padé approximation) for large values of velocity
 * and particularized to diatomic gas.
 */
class DART_API F0ElMachClamp : public F0ElSpeedSound
{
public:
    F0ElMachClamp(double _mInf, double _mC2 = 5) : F0ElSpeedSound(_mInf, _mC2) {}

    virtual double eval(tbox::Element const &e, std::vector<double> const &phi,
                        size_t k) const override;
    virtual double evalGrad(tbox::Element const &e, std::vector<double> const &phi,
                            size_t k) const override;
};

/**
 * @brief Linear Mach number (constant)
 */
class DART_API F0ElMachLin : public F0El
{
public:
    F0ElMachLin() : F0El() {}

    virtual double eval(tbox::Element const &e, std::vector<double> const &phi,
                        size_t k) const override;
    virtual double evalGrad(tbox::Element const &e, std::vector<double> const &phi,
                            size_t k) const override;
};

/**
 * @brief Isentropic pressure coefficient
 *
 * The pressure coefficient is particularized to diatomic gas.
 */
class DART_API F0ElCp : public F0El
{
    double gamma; ///< heat capacity ratio (diatomic gas only)
    double mInf;  ///< freestream Mach number

public:
    F0ElCp(double _mInf) : F0El(), gamma(1.4), mInf(_mInf) {}

    virtual double eval(tbox::Element const &e, std::vector<double> const &phi,
                        size_t k) const override;
    virtual double evalGrad(tbox::Element const &e, std::vector<double> const &phi,
                            size_t k) const override;
};

/**
 * @brief Isentropic clamped pressure coefficient
 *
 * The pressure coefficient is clamped (Padé approximation) for large values of
 * velocity and particularized to diatomic gas.
 */
class DART_API F0ElCpClamp : public F0ElSpeedSound
{
public:
    F0ElCpClamp(double _mInf, double _mC2 = 5) : F0ElSpeedSound(_mInf, _mC2) {}

    virtual double eval(tbox::Element const &e, std::vector<double> const &phi,
                        size_t k) const override;
    virtual double evalGrad(tbox::Element const &e, std::vector<double> const &phi,
                            size_t k) const override;
};

/**
 * @brief Linear pressure coefficient
 */
class DART_API F0ElCpLin : public F0El
{
public:
    F0ElCpLin() : F0El() {}

    virtual double eval(tbox::Element const &e, std::vector<double> const &phi,
                        size_t k) const override;
    virtual double evalGrad(tbox::Element const &e, std::vector<double> const &phi,
                            size_t k) const override;
};

} // namespace dart

#endif // WF0EL_H
