/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wKuttaElement.h"
#include "wNode.h"
using namespace tbox;
using namespace dart;

KuttaElement::KuttaElement(size_t _no, Element *&_surE, Element *&_volE,
                           std::vector<std::pair<size_t, Node *>> &_teN, int _sign)
    : wObject(), no(_no), surE(_surE), volE(_volE),
      teN(_teN), sign(_sign)

{
    // Sanity checks
    if (surE->type() == ElType::LINE2)
        nDim = 2;
    else if (surE->type() == ElType::TRI3)
        nDim = 3;
    else
    {
        std::stringstream err;
        err << "KuttaElement only implemented for surface elements of type "
            << ElType::LINE2 << " or " << ElType::TRI3
            << " (" << surE->type() << " was given)!\n";
        throw std::runtime_error(err.str());
    }
    if (volE->type() == ElType::TRI3 || volE->type() == ElType::TETRA4)
        nCol = volE->nodes.size();
    else
    {
        std::stringstream err;
        err << "KuttaElement only implemented for volume elements of type "
            << ElType::TRI3 << " or " << ElType::TETRA4
            << " (" << volE->type() << " was given)!\n";
        throw std::runtime_error(err.str());
    }
    // Count TE nodes
    mapNodes();
    nRow = teN.size();
}

/**
 * @brief Map the wake nodes to their local volume node indices
 */
void KuttaElement::mapNodes()
{
    // Find local volume index of contributing surface nodes
    for (auto p : teN)
    {
        for (size_t i = 0; i < volE->nodes.size(); ++i)
        {
            if (surE->nodes[p.first] == volE->nodes[i])
            {
                vsMap[p.first] = i;
                break;
            }
        }
    }
    // Check that all contributing surface nodes have been mapped
    if (vsMap.size() != teN.size())
    {
        std::stringstream err;
        err << "dart::KuttaElement: could not map some contributing nodes from surface element " << *surE << " to " << *volE << " (number of mapped nodes: " << vsMap.size() << ")!\n";
        throw std::runtime_error(err.str());
    }
}

void KuttaElement::write(std::ostream &out) const
{
    out << "KuttaElement #" << no
        << "\n\ton surface element (" << surE->type() << "): " << surE->no
        << "\n\ton volume element: (" << volE->type() << "): " << volE->no
        << "\n\twith " << teN.size() << " TE nodes: ";
    for (auto p : teN)
        out << p.second->no << " ";
    out << "\n\twith sign: " << sign << std::endl;
}