/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WKUTTAELEMENT_H
#define WKUTTAELEMENT_H

#include "dart.h"
#include "wObject.h"
#include "wElement.h"
#include <vector>
#include <map>

namespace dart
{

/**
 * @brief Super-element to apply Kutta condition
 *
 * Made up of one surface element at the trailing edge and the connected volume
 * element. Only Line2/Tri3 (surface) and Tri3/Tetra4 (volume) elements are
 * currently implemented.
 * Since Tri3 or Tetra4 are used, the evalutaion of the volume integrals is
 * performed at dummy Gauss point #0. If volume elements with non-constant shape
 * function derivatives are used, those derivates should be recomputed at the
 * true surface element Gauss points.
 * @authors Adrien Crovato
 */
class DART_API KuttaElement : public fwk::wObject
{
public:
    size_t no;           ///< ID
    tbox::Element *surE; ///< Surface element
    tbox::Element *volE; ///< Connected volume element

#ifndef SWIG
    size_t nRow; ///< number of rows in stiffness matrix
    size_t nCol; ///< number of columns in stiffness matrix
    size_t nDim; ///< dimension of volume element

    std::vector<std::pair<size_t, tbox::Node *>> teN; ///< Map of local node indices and trailing edge nodes
    std::map<size_t, size_t> vsMap;                   ///< map between surface node and volume node indices
    int sign;                                         ///< +1 if upper element, -1 if lower element

    KuttaElement(size_t _no, tbox::Element *&_surE, tbox::Element *&_volE, std::vector<std::pair<size_t, tbox::Node *>> &_teN, int _sign);

    virtual void write(std::ostream &out) const override;
#endif

private:
    void mapNodes();
};

} // namespace dart

#endif // WKUTTAELEMENT_H
