#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2020 University of Liège
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


## Compute the flow around the NASA CRM (Common Research Model) at CL = 0.5 and Mach 0.85
# CAD created by Guillem Battle I Capa
# Valid mesh obtained with gmsh 4.8.4

try:
   import tboxVtk
   import vtk
   hasVTK = True
except:
   hasVTK = False

def cfg():
    from fwk.wutils import parseargs
    import os.path
    # Mesh size
    wrms = 0.121 # wing root trailing edge mesh size
    wyms = 0.073 # wing root trailing edge mesh size
    wtms = 0.028 # wing root trailing edge mesh size
    trms = 0.055 # wing root trailing edge mesh size
    ttms = 0.023 # wing root trailing edge mesh size
    fums = 0.628 # fuselage mesh size
    fams = 36.8 # farfield mesh size
    # Parse
    args = parseargs()
    # Parameters
    return {
    # Options
    'Threads' : args.k, # number of threads
    'Verb' : args.verb + 2, # verbosity
    # Model (geometry or mesh)
    'File' : os.path.join(os.path.abspath(os.path.dirname(__file__)), '../models/crm.geo'), # Input file containing the model
    'Pars' : {'msWingR' : wrms, 'msWingY' : wyms, 'msWingT' : wtms,
              'msTailR' : trms, 'msTailT' : ttms,
              'msFus' : fums, 'msFar' : fams}, # parameters for input file model
    'Dim' : 3, # problem dimension (2 or 3)
    'Format' : 'vtk', # save format (vtk or gmsh)
    # Markers...
    'Fluid' : 'field', # name of physical group containing the fluid
    'Farfield' : ['upstream', 'farfield', 'downstream'], # LIST of names of physical groups containing the farfield boundaries (upstream/downstream should be first/last element)
    'Wings' : ['wing', 'tail'], # LIST of names of physical groups containing the lifting surface boundary (first element will be the body of interest for aerostructural and optimization)
    'Wakes' : ['wakeW', 'wakeT'], # LIST of names of physical group containing the wake
    'WakeTips' : ['wakeTipW', 'wakeTipT'], # LIST of names of physical group containing the edge of the wake
    'WakeExs' : ['wakeExW', 'wakeExT'], # LIST of names of physical group containing the edge of the wake and the intersection of lifting surface with fuselage (to be excluded from Wake B.C.)
    'Tes' : ['teW', 'teT'], # LIST of names of physical group containing the trailing edge
    'Fuselage' : 'fuselage', # name of physical group containing the fuselage boundaries
    'Symmetry' : 'symmetry', # name of physical group containing the symmetry boundaries
    # Freestream
    'M_inf' : 0.85, # freestream Mach number
    'AoA' : 1.2, # freestream angle of attack [deg] (optional, default=0)
    'AoS' : 0.0, # freestream angle of sideslip [deg] (optional, default=0)
    'Q_inf' : 0.0, # freesteam dynamic pressure (only required for aerostructural computations)
    # Geometry
    'S_ref' : 383.69/2, # reference surface length
    'c_ref' : 7.005, # reference chord length
    'x_ref' : 33.667, # x-coordinate of reference point for moment computation
    'y_ref' : 11.906, # y-coordinate of reference point for moment computation
    'z_ref' : 4.520, # z-coordinate of reference point for moment computation
    # Numerical
    'LSolver' : 'GMRES', # inner solver (PARDISO, MUMPS or GMRES)
    'G_fill' : 3, # fill-in factor for GMRES preconditioner (optional, default=2)
    'G_tol' : 1e-5, # tolerance for GMRES (optional, default=1e-5)
    'G_restart' : 50, # restart for GMRES (optional, default=50)
    'Rel_tol' : 1e-6, # relative tolerance on solver residual
    'Abs_tol' : 1e-8, # absolute tolerance on solver residual
    'Max_it' : 25 # maximum number of iterations for nonlinear solver
    }

def main():
    from dart.api.core import init_dart
    from dart import utils
    from fwk import Timers
    from fwk.testing import CTest, CTests
    import numpy as np
    import matplotlib.pyplot as plt
    # pre
    tms = Timers()
    tms['pre'].start()
    _dart = init_dart(cfg(), scenario='aerodynamic', task='analysis')
    msh, wrt, sol = (_dart.get(key) for key in ['msh', 'wrt', 'sol'])
    tms['pre'].stop()
    # solve
    tms['sol'].start()
    sol.run()
    sol.save(wrt)
    tms['sol'].stop()
    # post
    tms['pos'].start()
    if hasVTK:
        utils.write_slices(msh.name, [3.81973, 5.8765, 8.2271, 11.753, 14.6913, 17.6295, 19.0986, 21.4492, 24.9751, 27.91338], 12)
        for i in range(10):
            data = np.loadtxt(f'{msh.name}_slice_{i}.dat', delimiter=',', skiprows=2) # read
            plt.plot(data[:,0], data[:,-1], lw=2) # plot results
        font = {'family': 'serif', 'color': 'darkred', 'weight': 'normal', 'size': 16}
        plt.xlabel('$x/c$', fontdict=font)
        plt.ylabel('$C_p$', fontdict=font)
        plt.gca().invert_yaxis()
        plt.show()
    tms['pos'].stop()

    # display timers
    print('CPU statistics')
    print(tms)

    # check results
    tests = CTests()
    tests.add(CTest('CL', sol.Cl, 0.5105, 1e-1))
    tests.add(CTest('CD', sol.Cd, 0.0143, 1e-1))
    tests.add(CTest('CM', sol.Cm, -0.1214, 1e-1))
    tests.run()

    # eof
    print('')

if __name__ == '__main__':
    main()
